{% load render_table from django_tables2 %}
<div class="row">
    <div class="col col-md-12">
        <div class="card">
            <h5 class="card-header">
                <a href="{% url 'plugins:netbox_config_backup:backup_backups' pk=object.pk %}">Backups</a>
            </h5>
          <div class="card-body" id="object_list">
            {% include 'htmx/table.html' %}
          </div>
        </div>
    </div>
</div>